import time
from enum import Enum
from typing import List

from pydantic import BaseModel
from requests import Response

from cujirax.jira import Project
from cujirax.xray import Endpoint, get, login, post


class TestType(Enum):
    CUCUMBER = "Cucumber"
    MANUAL = "Manual"
    GENERIC = "Generic"


class Steps(BaseModel):
    action: str
    data: str = ""
    result: str = ""


class Fields(BaseModel):
    summary: str
    project: Project
    description: str = None
    

class TestCase(BaseModel):
    testtype: str
    fields: Fields
    

class CucumberTestCase(TestCase):
    testtype: str = TestType.CUCUMBER.value
    gherhin_def: str = None
    xray_test_sets: List[str] = None


class ManualTestCase(TestCase):
    testtype: str = TestType.MANUAL.value
    steps: Steps = None
    xray_test_sets: List[str] = None


class GenericTestCase(TestCase):
    testtype: str = TestType.GENERIC.value
    unstructured_def: str = None
    xray_test_repository_folder: str = None


def bulk_import(requestBody: List[TestCase], wait_until_success=True) -> Response:
    header = login()
    response =  post(Endpoint.CREATE_TEST_CASE.value, requestBody, header)
    print(response.status_code, response.json())
    if wait_until_success and response.status_code == 200:
        return check_status_retry(response.json().get('jobId'))
    return response


def check_status(job_id: str) -> Response:
    header = login()
    return get(Endpoint.CHECK_IMPORT_TEST_STATUS.value.format(job_id), header)

def check_status_retry(job_id: str) -> Response:
    max_retries = 60
    retry_interval_secs = 1
    
    for i in range(max_retries):
        response = check_status(job_id)
        status = response.json().get('status')
        if status == 'successful':
            return response
        else:
            print(f'Retry {i+1} - status: {status}')
            time.sleep(retry_interval_secs)
    
    raise Exception('Max retries exceeded - import test status not successful')




import os
import re
from atlassian import Jira
from typing import List
from pydantic import BaseModel


class Jirakey:
    value: str

    def __init__(self, value) -> None:
        self.value = value
        if not re.match(r'[A-Z][A-Z0-9_]*-[1-9][0-9]*$', value):
            raise ValueError('Invalid Jira key')

    def __repr__(self) -> str:
        return self.value


class Project(BaseModel):
    key: str


class IssueType(BaseModel):
    name: str


class Issue(BaseModel):
    summary: str
    project: Project
    issuetype: IssueType
    description: str


class JiraX(Jira):
    def __init__(self, key: str,
                 domain: str = None,
                 email: str = None,
                 secret: str = None
                 ):
        jira_domain = domain or os.getenv("JIRA_DOMAIN")
        jira_email = email or os.getenv("JIRA_EMAIL")
        jira_secret = secret or os.getenv("JIRA_SECRET")

        self.key = key
        super().__init__(
            url=f'https://{jira_domain}',
            username=jira_email,
            password=jira_secret)
        

    def get_testsets(self, summary: str) -> List[Jirakey]:
        return self.get_issues(summary=summary, type="Test Set")

    def get_testplan(self, summary: str) -> List[Jirakey]:
        return self.get_issues(summary=summary, type="Test Plan")

    def get_tests(self, summary: str) -> List[Jirakey]:
        return self.get_issues(summary=summary, type="Test")

    def get_issues(self, summary: str, type: str) -> List[Jirakey]:
        query = f'issuetype="{type}" AND summary~"{summary}" AND project="{self.key}"'
        issues = self.jql(query).get("issues")
        
        return [Jirakey(issue.get('key')) for issue in issues if issue.get('fields')['summary'] == summary]

    def link(self, parent_jira: str, child_jira: str, type="Parents"):
        return self.create_issue_link({
            "type": {"name": type},
            "inwardIssue": {"key": Jirakey(child_jira)},
            "outwardIssue": {"key": Jirakey(parent_jira)}
        })

    def create_issue(self, summary: str, description: str, issue_type: str) -> Jirakey:
        issue = self.get_issues(summary, issue_type)
        if issue:
            self.update_issue_field
            jira_key = issue[0]
            return jira_key

        issue = Issue(
            summary=summary,
            project=Project(key=self.key),
            issuetype=IssueType(name=issue_type),
            description=description)
        
        response = self.create_issue(fields=issue.dict())
        return Jirakey(response.get('key'))

    def create_testset(self, summary: str, description: str) -> Jirakey:
        return self.create_issue(summary, description, "Test Set")

    def create_testexecution(self, summary: str, description: str) -> Jirakey:
        return self.create_issue(summary, description, "Test Execution")

    def create_testplan(self, summary: str, description: str) -> Jirakey:
        return self.create_issue(summary, description, "Test Plan")
